__all__ = (
    "__title__",
    "__summary__",
    "__uri__",
    "__version__",
    "__author__ru__",
    "__author__en__",
    "__email__",
    "__license__",
    "__copyright__",
    "__maintainer__ru__",
    "__maintainer__en__",
    "__maintainer_email__",
)

__title__ = "OpenAV"
__summary__ = "OpenAV"
__uri__ = "https://github.com/DmitryRyumin/openav"

__version__ = "1.0"
__release__ = __version__ + ".0-a20"

__author__ru__ = "Рюмин Дмитрий, Иванько Денис, Шилов Николай, Маркитантов Максим, Карпов Алексей"
__author__en__ = "Dmitry Ryumin, Denis Ivanko, Nikolay Shilov, Maxim Markitantov, Alexey Karpov"
__email__ = (
    "dl_03.03.1991@mail.ru, denis.ivanko11@gmail.com, nick@iias.spb.su, m.markitantov@yandex.ru, " "karpov@iias.spb.su"
)

__maintainer__ru__ = "Рюмин Дмитрий"
__maintainer__en__ = "Dmitry Ryumin"
__maintainer_email__ = "dl_03.03.1991@mail.ru"

__license__ = "MIT"
__copyright__ = "2022-25. Dmitry Ryumin, Denis Ivanko, Nikolay Shilov, Maxim Markitantov, Alexey Karpov"
